/*
    triton -- ettercap plugin -- try to discover the gateway

    Copyright (C) 2001  ALoR <alor@users.sourceforge.net>
                        NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include "../../src/include/ec_main.h"
#include "../../src/include/ec_plugins.h"
#include "../../src/include/ec_inet_structures.h"
#include "../../src/include/ec_inet.h"
#include "../../src/include/ec_inet_forge.h"

char Ettercap_Version[] = VERSION;                       // required -- for compatibility

char PlugIn_Info[] = "Try to discover the LAN's gateway";

short PlugIn_Version = 13;


// ---------------------------


void PlugIn_Start(char**argv)
{
   int sock, MTU, len=0;
   char c[2] = "";
   char *buf;
   u_long MyIP, NetMask;
   ETH_header *eth;
   IP_header  *ip;
   char MAC[20];

   sock = Inet_OpenRawSock(Options.netiface);
   Inet_GetIfaceInfo(Options.netiface, &MTU, NULL, &MyIP, &NetMask);

   MyIP = MyIP & NetMask;

   buf = Inet_Forge_packet(MTU);
   fcntl(sock, F_SETFL, O_NONBLOCK);

   Plugin_Output("\nPassive searching of the gateway... (press return to exit)\n\n");

   loop
   {
      len = Inet_GetRawPacket(sock, buf, MTU, NULL);

      if (len > 0)
      {
         eth = (ETH_header *) buf;
         if (eth->type == htons(ETH_P_IP))
         {
            ip = (IP_header *)(eth+1);
            if ( (ip->dest_ip & NetMask) != MyIP || (ip->source_ip & NetMask) != MyIP )   // this is from the outer space... ;)
            {
               if ((ip->dest_ip & NetMask) != MyIP)
                  Inet_PutMACinString(MAC, eth->dest_mac);
               else if ((ip->source_ip & NetMask) != MyIP)
                  Inet_PutMACinString(MAC, eth->source_mac);

               if (number_of_hosts_in_lan > 1)
               {
                  int i;
                  for(i=0; i<number_of_hosts_in_lan; i++)
                     if (!strcmp(Host_In_LAN[i].mac, MAC))
                     {
                        Plugin_Output("Probably the gateway is %s --> %s\n", MAC, Host_In_LAN[i].ip );
                        break;
                     }
               }
               else
                  Plugin_Output("Probably the gateway is %s\n", MAC);
            }
         }
      }
      else
          usleep(1000);

      if (Plugin_Input(c, 1, P_NONBLOCK))
      {
         Inet_Forge_packet_destroy( buf );
         close(sock);
         return;
      }
   }

}

/* EOF */
